#define PANEL_PIN 11  //  The digital PWM pin that controls the LED panel.

// These defines are required to make things compatible with the Alnitak device, and hence Sequence Generator Pro.
// Please note however, this code had not been tested with Sequence Generator Pro.

//#define CMD_END chr(13)
//#define RESP_END chr(10)
//#define CMD_LENGTH 5
//#define ID "19"  //The device ID for an Alnitak FlatMan.
//#define VERSION "001"
//#define ERROR_COMMANDS false

// These defines are for my own device and are not compatible with Sequence Generator Pro.

#define CMD_END '<' //chr(13)
#define RESP_END '~' //chr(10)
#define CMD_LENGTH 5
#define ID "PL"  //A unique device ID for this panel.
#define VERSION "1.0"
#define ERROR_COMMANDS true

void ProcessCommand(char *buffer);
void ProcessError(char *buffer, char *error);
void ProcessPing(char *);
void ProcessOpen(char *);
void ProcessClose(char *);
void ProcessLightOn(char *);
void ProcessLightOff(char *);
void ProcessSetBrightness(char *);
void ProcessGetBrightness(char *);
void ProcessGetState(char *);
void ProcessGetVersion(char *);

/*
 * Class to represent the light panel.
 */
 
class TPanel {
  public:
    void SetBrightness(byte value);
    byte GetBrightness();
    void SwitchOn();
    void SwitchOff();
    bool GetState();
    TPanel(bool state, byte level, byte pin);
  private:  
    bool on;
    byte brightness;
    byte pin;
};

/*
 * TPanel member functions.
 */
 
void TPanel::SetBrightness(byte value) {
  brightness = value;
  if(on) {
    analogWrite(pin, brightness);
  }
}

byte TPanel::GetBrightness() {
  return brightness;
}

void TPanel::SwitchOn() {
  analogWrite(pin, brightness);
  on = true;
}

void TPanel::SwitchOff() {
  analogWrite(pin, 0);
  on = false;
}

bool TPanel::GetState() {
  return on;
}

TPanel::TPanel(bool state, byte level, byte ch) {
  on = state;
  brightness = level;
  pin = ch;
  pinMode(pin, OUTPUT);
  if(on) {
    analogWrite(pin, brightness);
  } else {
    analogWrite(pin, 0);
  }
}

/*
 * Create an instance of the Panel
 */
 
TPanel panel(false, 0, PANEL_PIN);

void setup() {
  Serial.begin(9600);
}

/*
 * Mainloop checks for serial data and processes it.
 */
 
void loop() {
  if(Serial.available()) {
    char buffer[15] = "";
    Serial.readBytesUntil(CMD_END, buffer, 14);
    ProcessInput(buffer);
  }
}

/*
 * Check that the command has the correct leading character
 * and is the correct length.
 */
 
void ProcessInput(char *buffer) {
  if(buffer[0] != '>' || strlen(buffer) != CMD_LENGTH) {  // This is not a command so parse it as an error.
    ProcessError(buffer, "Command is not correctly formed.");
  } else {
    ProcessCommand(buffer);
  }
}

/*
 * Determine the command type and call the appropriate function.
 */
 
void ProcessCommand(char *buffer) {
    switch(buffer[1]) {
      case 'P': 
       ProcessPing(buffer);
        break;
      case 'O': 
       ProcessOpen(buffer);
        break;
      case 'C': 
       ProcessClose(buffer);
        break;
      case 'L': 
       ProcessLightOn(buffer);
        break;
       case 'D': 
       ProcessLightOff(buffer);
        break;
       case 'B': 
       ProcessSetBrightness(buffer);
        break;
      case 'J': 
       ProcessGetBrightness(buffer);
        break;
      case 'S': 
       ProcessGetState(buffer);
        break;
      case 'V': 
       ProcessGetVersion(buffer);
        break;
    }
}

/*
 * Processing for when an error occurs.
 */
 
void ProcessError(char *buffer, char *errorText) {
  if(ERROR_COMMANDS) {
    Serial.print("*E");
    Serial.print(buffer);
    Serial.print(errorText);
    Serial.print("~");
  }
}

/*
 * Send the passed response back over the serial port.
 */
 
void SendResponse(char *response) {
  Serial.print(response);
}

/*
 * Functions to process each of the commands.
 */

void ProcessPing(char *cmd) {
  char response[15] = "";
  sprintf(response, "*P%2sOOO%c", ID, RESP_END);
  SendResponse(response);
}

void ProcessOpen(char *cmd) {
  char response[15] = "";
  sprintf(response, "*O%2sOOO%c", ID, RESP_END);
  SendResponse(response);
}

void ProcessClose(char *cmd) {
  char response[15] = "";
  sprintf(response, "*C%2sOOO%c", ID, RESP_END);
  SendResponse(response);
}

void ProcessLightOn(char *cmd) {
  char response[15] = "";
  panel.SwitchOn();
  sprintf(response, "*L%2sOOO%c", ID, RESP_END);
  SendResponse(response);
}

void ProcessLightOff(char *cmd) {
  char response[15] = "";
  panel.SwitchOff();
  sprintf(response, "*D%2sOOO%c", ID, RESP_END);
  SendResponse(response);
}

void ProcessSetBrightness(char *cmd) {
  char response[15] = "";
  bool error = false;
  byte value = 0;
  for(int i=2;i<5;i++) {
    if(!isDigit(cmd[i])) {
      error = true;
    }
  }
  if(!error) {
    value = 100*(cmd[2]-'0') + 10*(cmd[3]-'0') + (cmd[4]-'0');
    if(value > 255) {
      ProcessError(cmd, "Brightness exceeds limit (255).");
    }
    panel.SetBrightness(value);
  } else {
    ProcessError(cmd, "Invalid brightness level");
  }
  sprintf(response, "*B%2s%03d%c", ID, value, RESP_END);
  SendResponse(response);
}

void ProcessGetBrightness(char *cmd) {
  char response[15] = "";
  byte value = panel.GetBrightness();
  sprintf(response, "*J%2s%03d%c", ID, value, RESP_END);
  SendResponse(response);
}

void ProcessGetState(char *cmd) {
  char response[15] = "";
  sprintf(response, "*S%2s%1d%1d%1d%c", ID, 0, panel.GetState()?1:0, 0, RESP_END);
  SendResponse(response);
}

void ProcessGetVersion(char *cmd) {
  char response[15] = "";
  sprintf(response, "*V%2s%3s%c", ID, VERSION, RESP_END);
  SendResponse(response);
}


